@@ -4,7 +4,7 @@
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### Constructor 
+### Constructor
 ###
 
 .DoparParam_prototype <- .BiocParallelParam_prototype
@@ -62,20 +62,28 @@ setMethod("bpisup", "DoparParam",
 ###
 
 setMethod("bplapply", c("ANY", "DoparParam"),
-    function(X, FUN, ..., BPREDO=list(), BPPARAM=bpparam())
+    function(X, FUN, ...,
+             BPREDO=list(), BPPARAM=bpparam(), BPOPTIONS = bpoptions())
 {
     if (!length(X))
         return(.rename(list(), X))
 
     FUN <- match.fun(FUN)
+    BPREDO <- bpresult(BPREDO)
 
     idx <- .redo_index(X, BPREDO)
-    if (any(idx))
+    if (length(idx))
         X <- X[idx]
 
+    OPTIONS <- .workerOptions(
+        log = bplog(BPPARAM),
+        stop.on.error = bpstopOnError(BPPARAM),
+        timeout = bptimeout(BPPARAM),
+        exportglobals = bpexportglobals(BPPARAM)
+    )
+
     FUN <- .composeTry(
-        FUN, bplog(BPPARAM), bpstopOnError(BPPARAM),
-        timeout=bptimeout(BPPARAM), exportglobals=bpexportglobals(BPPARAM)
+        FUN, OPTIONS = OPTIONS, SEED = NULL
     )
 
     i <- NULL
@@ -100,19 +108,20 @@ setMethod("bplapply", c("ANY", "DoparParam"),
 
     names(res) <- names(X)
 
-    if (any(idx)) {
+    if (length(BPREDO) && length(idx)) {
         BPREDO[idx] <- res
-        res <- BPREDO 
+        res <- BPREDO
     }
 
-    if (!all(bpok(res)))
+    if (!.bpallok(res))
         stop(.error_bplist(res))
 
     res
 })
 
 setMethod("bpiterate", c("ANY", "ANY", "DoparParam"),
-    function(ITER, FUN, ..., BPPARAM=bpparam())
+    function(ITER, FUN, ..., BPREDO = list(),
+             BPPARAM=bpparam(), BPOPTIONS=bpoptions())
 {
     stop("'bpiterate' not supported for DoparParam")
 })
