@@ -25,10 +25,32 @@
     if (identical(tolower(port), "random") || is.na(port)) {
         .rng_internal_stream$set()
         on.exit(.rng_internal_stream$reset())
-        port <- as.integer(
-            11000 +
-            1000 * ((stats::runif(1L) + unclass(Sys.time()) / 300) %% 1L)
-        )
+        portAvailable <- FALSE
+        for (i in 1:5) {
+            port <- as.integer(
+                11000 +
+                1000 * ((stats::runif(1L) + unclass(Sys.time()) / 300) %% 1L)
+            )
+            tryCatch(
+                {
+                    ## User should not be able to interrupt the port check
+                    ## Otherwise we might have an unclosed connection
+                    suspendInterrupts(
+                        {
+                            con <- serverSocket(port)
+                            close(con)
+                        }
+                    )
+                    portAvailable <- TRUE
+                },
+                error = function(e) {
+                    message("failed to open the port ", port,", trying a new port...")
+                }
+            )
+            if (portAvailable) break
+        }
+        if (!portAvailable)
+            .stop("cannot find an open port. For manually specifying the port, see ?SnowParam")
     } else {
         port <- as.integer(port)
     }
@@ -100,11 +122,12 @@ SnowParam <- function(workers=snowWorkers(type),
                       type=c("SOCK", "MPI", "FORK"), tasks=0L,
                       stop.on.error=TRUE,
                       progressbar=FALSE, RNGseed=NULL,
-                      timeout=30L * 24L * 60L * 60L,
-                      exportglobals=TRUE,
+                      timeout=WORKER_TIMEOUT,
+                      exportglobals=TRUE, exportvariables=TRUE,
                       log=FALSE, threshold="INFO", logdir=NA_character_,
                       resultdir=NA_character_, jobname = "BPJOB",
                       force.GC = FALSE,
+                      fallback = TRUE,
                       manager.hostname=NA_character_,
                       manager.port=NA_integer_, ...)
 {
@@ -117,7 +140,7 @@ SnowParam <- function(workers=snowWorkers(type),
         stop("'workers' must be integer(1) when 'type' is MPI or FORK")
 
     if (progressbar && missing(tasks))
-        tasks <- .Machine$integer.max
+        tasks <- TASKS_MAXIMUM
 
     clusterargs <- c(list(spec=workers, type=type), list(...))
 
@@ -142,10 +165,13 @@ SnowParam <- function(workers=snowWorkers(type),
         .controlled=TRUE, workers=workers, tasks=as.integer(tasks),
         stop.on.error=stop.on.error,
         progressbar=progressbar, RNGseed=RNGseed,
-        timeout=as.integer(timeout), exportglobals=exportglobals,
+        timeout=as.integer(timeout),
+        exportglobals=exportglobals,
+        exportvariables=exportvariables,
         log=log, threshold=threshold, logdir=logdir,
         resultdir=resultdir, jobname=jobname,
         force.GC = force.GC,
+        fallback = fallback,
         hostname=manager.hostname, port=manager.port,
         ...
     )
@@ -236,17 +262,32 @@ setMethod("bpstart", "SnowParam",
 
     ## FORK (useRscript not relevant)
     if (x$.clusterargs$type == "FORK") {
-        bpbackend(x) <- .bpfork(nnodes, bptimeout(x), .hostname(x), .port(x))
+        socket_timeout <- as.integer(getOption("timeout"))
+        socket_timeout_is_valid <-
+            length(socket_timeout) == 1L && !is.na(socket_timeout) &&
+            socket_timeout > 0L
+        if (!socket_timeout_is_valid)
+            stop("'getOption(\"timeout\")' must be positive integer(1)")
+        bpbackend(x) <- .bpfork(nnodes, .hostname(x), .port(x), socket_timeout)
     ## SOCK, MPI
     } else {
         cargs <- x$.clusterargs
         cargs$spec <- if (is.numeric(cargs$spec)) {
             nnodes
         } else cargs$spec[seq_len(nnodes)]
+
+        ## work around devtools::load_all()
+        ##
+        ## 'inst' exists when using devtools::load_all()
+        libPath <- find.package("BiocParallel")
+        if (dir.exists(file.path(libPath, "inst")))
+            libPath <- file.path(libPath, "inst")
+
         if (is.null(cargs$snowlib))
-            cargs$snowlib <- find.package("BiocParallel")
+            cargs$snowlib <- libPath
+
         if (!is.null(cargs$useRscript) && !cargs$useRscript)
-            cargs$scriptdir <- find.package("BiocParallel")
+            cargs$scriptdir <- libPath
 
         if (x$.clusterargs$type == "SOCK") {
             cargs$master <- .hostname(x)
@@ -368,3 +409,68 @@ setAs("spawnedMPIcluster", "SnowParam",
     .SnowParam(.clusterargs=.clusterargs, cluster=from, .controlled=FALSE,
                workers=length(from))
 })
+
+### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+### task dispatching interface
+###
+
+setOldClass(c("SOCK0node", "SOCKnode")) # needed for method dispatch
+
+.SOCKmanager <- setClass("SOCKmanager", contains = "TaskManager")
+
+setMethod(
+    ".manager", "SnowParam",
+    function(BPPARAM)
+{
+    manager <- callNextMethod()
+    manager$initialized <- rep(FALSE, manager$capacity)
+    as(manager, "SOCKmanager")
+})
+
+setMethod(
+    ".manager_send", "SOCKmanager",
+    function(manager, value, ...)
+{
+    availability <- manager$availability
+    stopifnot(length(availability) >= 0L)
+    ## send the job to the next available worker
+    worker <- names(availability)[1]
+    id <- as.integer(worker)
+    ## Do the cache only when the snow worker is
+    ## created by our package.
+    if (.controlled(manager$BPPARAM)) {
+        if (manager$initialized[id])
+            value <- .task_dynamic(value)
+        else
+            manager$initialized[id] <- TRUE
+    }
+    .send_to(manager$backend, as.integer(worker), value)
+    rm(list = worker, envir = availability)
+    manager
+})
+
+setMethod(
+    ".manager_cleanup", "SOCKmanager",
+    function(manager)
+{
+    manager <- callNextMethod()
+    manager$initialized <- rep(FALSE, manager$capacity)
+    if (.controlled(manager$BPPARAM)) {
+        value <- .EXEC(tag = NULL, .clean_task_static, args = NULL)
+        .send_all(manager$backend, value)
+        msg <- .recv_all(manager$backend)
+    }
+    manager
+})
+
+## The worker class of SnowParam
+setMethod(
+    ".recv", "SOCKnode",
+    function(worker)
+{
+    msg <- callNextMethod()
+    if (inherits(msg, "error"))
+        return(msg)
+    ## read/write the static value(if any)
+    .load_task_static(msg)
+})
