@@ -5,17 +5,19 @@
 ## All params have dedicated bpiterate() methods.
 
 setMethod("bpiterate", c("ANY", "ANY", "missing"),
-    function(ITER, FUN, ..., BPPARAM=bpparam())
+    function(ITER, FUN, ..., BPREDO=list(),
+             BPPARAM=bpparam(), BPOPTIONS=bpoptions())
 {
     ITER <- match.fun(ITER)
     FUN <- match.fun(FUN)
-    bpiterate(ITER, FUN, ..., BPPARAM=BPPARAM)
+    bpiterate(ITER, FUN, ..., BPREDO = BPREDO,
+              BPPARAM=BPPARAM, BPOPTIONS = BPOPTIONS)
 })
 
 ## TODO: support BPREDO
 .bpiterate_impl <-
     function(ITER, FUN, ..., REDUCE, init, reduce.in.order = FALSE,
-             BPPARAM = bpparam())
+              BPREDO = list(), BPPARAM = bpparam(), BPOPTIONS=bpoptions())
 {
     ## Required API
     ##
@@ -25,13 +27,7 @@ setMethod("bpiterate", c("ANY", "ANY", "missing"),
     ITER <- match.fun(ITER)
     FUN <- match.fun(FUN)
 
-    ITER_ <- function() {
-        list(ITER())
-    }
-
     if (missing(REDUCE)) {
-        if (reduce.in.order)
-            stop("REDUCE must be provided when 'reduce.in.order = TRUE'")
         if (!missing(init))
             stop("REDUCE must be provided when 'init' is given")
     }
@@ -39,18 +35,16 @@ setMethod("bpiterate", c("ANY", "ANY", "missing"),
     ARGS <- list(...)
 
     manager <- structure(list(), class="iterate") # dispatch
-    res <- .bpinit(
+    .bpinit(
         manager = manager,
-        ITER = ITER_,
+        ITER = ITER,
         FUN = FUN,
         ARGS = ARGS,
         BPPARAM = BPPARAM,
+        BPOPTIONS = BPOPTIONS,
+        BPREDO = BPREDO,
         init = init,
         REDUCE = REDUCE,
         reduce.in.order = reduce.in.order
     )
-
-    ## TODO: handle the error in res
-
-    res
 }
