@@ -68,11 +68,13 @@ setMethod("bpstart", "missing",
 .bpstart_set_logging <-
     function(x)
 {
-    cluster <- bpbackend(x)
+    manager <- .manager(x)
+    on.exit({.manager_cleanup(manager)})
 
     value <- .EXEC(NULL, .log_load, list(bplog(x), bpthreshold(x), TRUE))
-    .send_all(cluster, value)
-    response <- .recv_all(cluster)
+    .manager_send_all(manager, value)
+    .manager_flush(manager)
+    response <- .manager_recv_all(manager)
 
     .bpstart_error_handler(x, response, "set_logging")
     invisible(x)
