@@ -65,96 +65,39 @@
     }
 }
 
-.split_X_redo <-
-    function(X, redo_index, elements_per_task, n)
+.ntask <-
+    function(X, workers, tasks)
 {
-    ## Whether we need to calculate n?
-    prealloc <- missing(n)
-    splittedX <- vector("list", ifelse(prealloc, 0L, n))
-    last_redo_status <- redo_index[[1]]
-    task_i <- 1L
-    x_start <- 1L
-    x_len <- 0L
-    for (i in seq_along(redo_index)) {
-        redo <- redo_index[[i]]
-        ## The task division depends on two factors:
-        ## 1. Whether the current redo is different from the previous one?
-        ## 2. Whether the elements number in a task will exceed
-        ##    `elements_per_task` limit
-        is_switch <- xor(last_redo_status, redo)
-        is_switch <- ifelse(
-            is_switch || !redo, is_switch, x_len >= elements_per_task
-        )
-        ## If we want to split X at this position and it is not in the
-        ## preallocation stage, we create a subset of X or
-        ## .bploop_rng_iter based on the previous redo value
-        if (!prealloc && is_switch) {
-            if (last_redo_status)
-                splittedX[[task_i]] <- X[seq.int(x_start, length.out = x_len)]
-            else
-                splittedX[[task_i]] <- .bploop_rng_iter(x_len)
-        }
-        ## if we split X, record the next redo status, start index,
-        ## add task index by 1 and reset the length, otherwise we just
-        ## add the length by 1
-        if (is_switch) {
-            x_start <- ifelse(last_redo_status, x_start + x_len, x_start)
-            last_redo_status <- redo
-            task_i <- task_i + 1L
-            x_len <- 1L
-        } else {
-            x_len <- x_len + 1
-        }
-        if (!prealloc && task_i >= n)
-            break
-    }
-    ## The last task does not have the "switch" so we need to manually add it
-    if (!prealloc && last_redo_status) {
-        splittedX[[task_i]] <- X[seq.int(x_start, length(X))]
-    }
-
-    if (prealloc) {
-        task_i - !last_redo_status
+    if (is.na(tasks)) {
+        length(X)
+    } else if (tasks == 0L) {
+        workers
     } else {
-        splittedX
+        min(length(X), tasks)
     }
 }
 
 .splitX <-
-    function(X, workers, tasks, redo_index = NULL)
+    function(X, workers, tasks)
 {
-    if (tasks == 0L) {
-        tasks <- workers
-    } else {
-        tasks <- min(length(X), tasks)
-    }
-    ## If redo index presents, split X based on the index while
-    ## preserving the correct rng stream.  it will respect `tasks`
-    ## value
-    if (length(redo_index)) {
-        ## Two-pass algorithm, the first pass calculate the required
-        ## list size to allocate, then do the real allocation
-        elements_per_task <- ceiling(length(X) / max(tasks, 1L))
-        n <- .split_X_redo(X, redo_index, elements_per_task)
-        .split_X_redo(X, redo_index, elements_per_task, n)
-    } else {
-        idx <- .splitIndices(length(X), tasks)
-        relist(X, idx)
-    }
-
+    tasks <- .ntask(X, workers, tasks)
+    idx <- .splitIndices(length(X), tasks)
+    relist(X, idx)
 }
 
 .redo_index <-
     function(X, BPREDO)
 {
-    idx <- !bpok(BPREDO)
-    if (length(idx)) {
+    if (length(BPREDO)) {
         if (length(BPREDO) != length(X))
             stop("'length(BPREDO)' must equal 'length(X)'")
-        if (!any(idx))
+        idx <- which(!bpok(BPREDO))
+        if (!length(idx))
             stop("no previous error in 'BPREDO'")
+        idx
+    } else {
+        seq_along(X)
     }
-    idx
 }
 
 ## re-apply names on X of lapply(X, FUN) to the return value
@@ -248,3 +191,15 @@
     )
     stop(msg, call. = FALSE)
 }
+
+## BatchJobs / batchtools signal no timeout with 'Inf', rather than
+## NA; do not implement as bptimeout() method because NA is
+## appropriate in other contexts, e.g., when 'show()'ing param.
+.batch_bptimeout <-
+    function(BPPARAM)
+{
+    timeout <- bptimeout(BPPARAM)
+    if (identical(timeout, NA_integer_))
+        timeout <- Inf
+    timeout
+}
