@@ -16,4 +16,10 @@ test_BiocParallelParam <-
 
     ## non-default inherited slot
     checkIdentical("WARN", bpthreshold(A(threshold = "WARN")))
+
+    ## workers (specified as character()) more than tasks
+    checkException(
+        validObject(A(workers = rep("a", 3L), tasks = 2L)),
+        silent = TRUE
+    )
 }
